@@ -1,7 +1,8 @@
-/* This code is derived from the Linux Kernel sources intended
+/* SPDX-License-Identifier: GPL-2.0-only
+ *
+ * This code is derived from the Linux Kernel sources intended
  * to maintain compatibility with different Kernel versions.
  * Copyright of original source is of respective Linux Kernel authors.
- * License is GPL-2.0-only.
  */
 
 #ifndef COMPAT_NETFLOW_H
@@ -695,8 +696,8 @@ static inline int is_vlan_dev(struct net_device *dev)
 }
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0)
-# ifdef CONFIG_BRIDGE_NETFILTER
+#ifdef CONFIG_BRIDGE_NETFILTER
+# ifndef HAVE_NF_BRIDGE_INFO_GET
 static inline struct nf_bridge_info *
 nf_bridge_info_get(const struct sk_buff *skb)
 {
@@ -749,4 +750,29 @@ unsigned long long strtoul(const char *cp, char **endp, unsigned int base)
 	return result;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,12,0)
+/*
+ * find_module() is unexported in v5.12:
+ *   089049f6c9956 ("module: unexport find_module and module_mutex")
+ * and module_mutex is replaced with RCU in
+ *   a006050575745 ("module: use RCU to synchronize find_module")
+ */
+#include <linux/rcupdate.h>
+struct module *find_module(const char *name)
+{
+	struct module *mod;
+
+	rcu_read_lock_sched();
+	/* Yes this is crazy, but should work. */
+	list_for_each_entry_rcu(mod, &THIS_MODULE->list, list) {
+		if (!strcmp(mod->name, name)) {
+			rcu_read_unlock_sched();
+			return mod;
+		}
+	}
+	rcu_read_unlock_sched();
+	return NULL;
+}
+#endif
+
 #endif /* COMPAT_NETFLOW_H */
